@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 #
 # httpresponse.rb -- HTTPResponse Class
 #
@@ -15,16 +16,82 @@ require 'webrick/httputils'
 require 'webrick/httpstatus'
 
 module WEBrick
+  ##
+  # An HTTP response.  This is filled in by the service or do_* methods of a
+  # WEBrick HTTP Servlet.
+
   class HTTPResponse
-    attr_reader :http_version, :status, :header
+
+    ##
+    # HTTP Response version
+
+    attr_reader :http_version
+
+    ##
+    # Response status code (200)
+
+    attr_reader :status
+
+    ##
+    # Response header
+
+    attr_reader :header
+
+    ##
+    # Response cookies
+
     attr_reader :cookies
+
+    ##
+    # Response reason phrase ("OK")
+
     attr_accessor :reason_phrase
+
+    ##
+    # Body may be a String or IO-like object that responds to #read and
+    # #readpartial.
+
     attr_accessor :body
 
-    attr_accessor :request_method, :request_uri, :request_http_version
+    ##
+    # Request method for this response
+
+    attr_accessor :request_method
+
+    ##
+    # Request URI for this response
+
+    attr_accessor :request_uri
+
+    ##
+    # Request HTTP version for this response
+
+    attr_accessor :request_http_version
+
+    ##
+    # Filename of the static file in this response.  Only used by the
+    # FileHandler servlet.
+
     attr_accessor :filename
+
+    ##
+    # Is this a keep-alive response?
+
     attr_accessor :keep_alive
-    attr_reader :config, :sent_size
+
+    ##
+    # Configuration for this response
+
+    attr_reader :config
+
+    ##
+    # Bytes sent in this response
+
+    attr_reader :sent_size
+
+    ##
+    # Creates a new HTTP response object.  WEBrick::Config::HTTP is the
+    # default configuration.
 
     def initialize(config)
       @config = config
@@ -45,58 +112,97 @@ module WEBrick
       @sent_size = 0
     end
 
+    ##
+    # The response's HTTP status line
+
     def status_line
       "HTTP/#@http_version #@status #@reason_phrase #{CRLF}"
     end
 
+    ##
+    # Sets the response's status to the +status+ code
+
     def status=(status)
       @status = status
       @reason_phrase = HTTPStatus::reason_phrase(status)
     end
 
+    ##
+    # Retrieves the response header +field+
+
     def [](field)
       @header[field.downcase]
     end
 
+    ##
+    # Sets the response header +field+ to +value+
+
     def []=(field, value)
       @header[field.downcase] = value.to_s
     end
 
+    ##
+    # The content-length header
+
     def content_length
       if len = self['content-length']
         return Integer(len)
       end
     end
 
+    ##
+    # Sets the content-length header to +len+
+
     def content_length=(len)
       self['content-length'] = len.to_s
     end
 
+    ##
+    # The content-type header
+
     def content_type
       self['content-type']
     end
 
+    ##
+    # Sets the content-type header to +type+
+
     def content_type=(type)
       self['content-type'] = type
     end
 
+    ##
+    # Iterates over each header in the response
+
     def each
-      @header.each{|k, v|  yield(k, v) }
+      @header.each{|field, value|  yield(field, value) }
     end
 
+    ##
+    # Will this response body be returned using chunked transfer-encoding?
+
     def chunked?
       @chunked
     end
 
+    ##
+    # Enables chunked transfer encoding.
+
     def chunked=(val)
       @chunked = val ? true : false
     end
 
+    ##
+    # Will this response's connection be kept alive?
+
     def keep_alive?
       @keep_alive
     end
 
-    def send_response(socket)
+    ##
+    # Sends the response on +socket+
+
+    def send_response(socket) # :nodoc:
       begin
         setup_header()
         send_header(socket)
@@ -110,7 +216,10 @@ module WEBrick
       end
     end
 
-    def setup_header()
+    ##
+    # Sets up the headers for sending
+
+    def setup_header() # :nodoc:
       @reason_phrase    ||= HTTPStatus::reason_phrase(@status)
       @header['server'] ||= @config[:ServerSoftware]
       @header['date']   ||= Time.now.httpdate
@@ -142,7 +251,7 @@ module WEBrick
         @header.delete('content-length')
       elsif @header['content-length'].nil?
         unless @body.is_a?(IO)
-          @header['content-length'] = @body ? @body.size : 0
+          @header['content-length'] = @body ? @body.bytesize : 0
         end
       end
 
@@ -150,8 +259,13 @@ module WEBrick
       if @header['connection'] == "close"
          @keep_alive = false
       elsif keep_alive?
-        if chunked? || @header['content-length']
+        if chunked? || @header['content-length'] || @status == 304 || @status == 204 || HTTPStatus.info?(@status)
           @header['connection'] = "Keep-Alive"
+        else
+          msg = "Could not determine content-length of response body. Set content-length of the response or set Response#chunked = true"
+          @logger.warn(msg)
+          @header['connection'] = "close"
+          @keep_alive = false
         end
       else
         @header['connection'] = "close"
@@ -165,7 +279,10 @@ module WEBrick
       end
     end
 
-    def send_header(socket)
+    ##
+    # Sends the headers on +socket+
+
+    def send_header(socket) # :nodoc:
       if @http_version.major > 0
         data = status_line()
         @header.each{|key, value|
@@ -180,35 +297,51 @@ module WEBrick
       end
     end
 
-    def send_body(socket)
-      case @body
-      when IO then send_body_io(socket)
-      else send_body_string(socket)
+    ##
+    # Sends the body on +socket+
+
+    def send_body(socket) # :nodoc:
+      if @body.respond_to? :readpartial then
+        send_body_io(socket)
+      elsif @body.respond_to?(:call) then
+        send_body_proc(socket)
+      else
+        send_body_string(socket)
       end
     end
 
-    def to_s
+    def to_s # :nodoc:
       ret = ""
       send_response(ret)
       ret
     end
 
+    ##
+    # Redirects to +url+ with a WEBrick::HTTPStatus::Redirect +status+.
+    #
+    # Example:
+    #
+    #   res.set_redirect WEBrick::HTTPStatus::TemporaryRedirect
+
     def set_redirect(status, url)
-      @body = "<HTML><A HREF=\"#{url.to_s}\">#{url.to_s}</A>.</HTML>\n"
+      @body = "<HTML><A HREF=\"#{url}\">#{url}</A>.</HTML>\n"
       @header['location'] = url.to_s
       raise status
     end
 
+    ##
+    # Creates an error page for exception +ex+ with an optional +backtrace+
+
     def set_error(ex, backtrace=false)
       case ex
-      when HTTPStatus::Status 
+      when HTTPStatus::Status
         @keep_alive = false if HTTPStatus::error?(ex.code)
         self.status = ex.code
-      else 
+      else
         @keep_alive = false
         self.status = HTTPStatus::RC_INTERNAL_SERVER_ERROR
       end
-      @header['content-type'] = "text/html"
+      @header['content-type'] = "text/html; charset=ISO-8859-1"
 
       if respond_to?(:create_error_page)
         create_error_page()
@@ -221,6 +354,14 @@ module WEBrick
         host, port = @config[:ServerName], @config[:Port]
       end
 
+      error_body(backtrace, ex, host, port)
+    end
+
+    private
+
+    # :stopdoc:
+
+    def error_body(backtrace, ex, host, port)
       @body = ''
       @body << <<-_end_of_html_
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN">
@@ -250,21 +391,23 @@ module WEBrick
       _end_of_html_
     end
 
-    private
-
     def send_body_io(socket)
       begin
         if @request_method == "HEAD"
           # do nothing
         elsif chunked?
-          while buf = @body.read(@buffer_size)
-            next if buf.empty?
-            data = ""
-            data << format("%x", buf.size) << CRLF
-            data << buf << CRLF
+          buf  = ''
+          begin
+            @body.readpartial(@buffer_size, buf)
+            size = buf.bytesize
+            data = "#{size.to_s(16)}#{CRLF}#{buf}#{CRLF}"
             _write_data(socket, data)
-            @sent_size += buf.size
-          end
+            data.clear
+            @sent_size += size
+          rescue EOFError
+            break
+          end while true
+          buf.clear
           _write_data(socket, "0#{CRLF}#{CRLF}")
         else
           size = @header['content-length'].to_i
@@ -280,29 +423,62 @@ module WEBrick
       if @request_method == "HEAD"
         # do nothing
       elsif chunked?
-        remain = body ? @body.size : 0
+        body ? @body.bytesize : 0
         while buf = @body[@sent_size, @buffer_size]
           break if buf.empty?
-          data = ""
-          data << format("%x", buf.size) << CRLF
-          data << buf << CRLF
+          size = buf.bytesize
+          data = "#{size.to_s(16)}#{CRLF}#{buf}#{CRLF}"
+          buf.clear
           _write_data(socket, data)
-          @sent_size += buf.size
+          @sent_size += size
         end
         _write_data(socket, "0#{CRLF}#{CRLF}")
       else
-        if @body && @body.size > 0
+        if @body && @body.bytesize > 0
           _write_data(socket, @body)
-          @sent_size = @body.size
+          @sent_size = @body.bytesize
         end
       end
     end
 
+    def send_body_proc(socket)
+      if @request_method == "HEAD"
+        # do nothing
+      elsif chunked?
+        @body.call(ChunkedWrapper.new(socket, self))
+        _write_data(socket, "0#{CRLF}#{CRLF}")
+      else
+        size = @header['content-length'].to_i
+        @body.call(socket)
+        @sent_size = size
+      end
+    end
+
+    class ChunkedWrapper
+      def initialize(socket, resp)
+        @socket = socket
+        @resp = resp
+      end
+
+      def write(buf)
+        return if buf.empty?
+        socket = @socket
+        @resp.instance_eval {
+          size = buf.bytesize
+          data = "#{size.to_s(16)}#{CRLF}#{buf}#{CRLF}"
+          _write_data(socket, data)
+          data.clear
+          @sent_size += size
+        }
+      end
+      alias :<< :write
+    end
+
     def _send_file(output, input, offset, size)
       while offset > 0
         sz = @buffer_size < size ? @buffer_size : size
         buf = input.read(sz)
-        offset -= buf.size
+        offset -= buf.bytesize
       end
 
       if size == 0
@@ -314,7 +490,7 @@ module WEBrick
           sz = @buffer_size < size ? @buffer_size : size
           buf = input.read(sz)
           _write_data(output, buf)
-          size -= buf.size
+          size -= buf.bytesize
         end
       end
     end
@@ -322,5 +498,8 @@ module WEBrick
     def _write_data(socket, data)
       socket << data
     end
+
+    # :startdoc:
   end
+
 end
