@@ -1,6 +1,9 @@
+# frozen_string_literal: false
 require 'drb/drb'
 require 'drb/extserv'
 
+module DRbTests
+
 class Foo
   include DRbUndumped
 end
@@ -11,12 +14,14 @@ class Bar
     @foo = Foo.new
   end
   attr_reader :foo
-  
+
   def foo?(foo)
     @foo == foo
   end
 end
 
+end
+
 if __FILE__ == $0
   def ARGV.shift
     it = super()
@@ -24,7 +29,9 @@ if __FILE__ == $0
     it
   end
 
-  DRb.start_service('druby://:0', Bar.new)
+  DRb.start_service('druby://localhost:0', DRbTests::Bar.new)
   es = DRb::ExtServ.new(ARGV.shift, ARGV.shift)
   DRb.thread.join
+  es.stop_service if es.alive?
 end
+
