@@ -1,22 +1,10 @@
-require_relative 'helper'
+# frozen_string_literal: true
+begin
+  require_relative 'helper'
+rescue LoadError
+end
 
 class TestFiddle < Fiddle::TestCase
-  def test_constants_match
-    [
-      :TYPE_VOID,
-      :TYPE_VOIDP,
-      :TYPE_CHAR,
-      :TYPE_SHORT,
-      :TYPE_INT,
-      :TYPE_LONG,
-      :TYPE_LONG_LONG,
-      :TYPE_FLOAT,
-      :TYPE_DOUBLE,
-    ].each do |name|
-      assert_equal(DL.const_get(name), Fiddle.const_get(name))
-    end
-  end
-
   def test_windows_constant
     require 'rbconfig'
     if RbConfig::CONFIG['host_os'] =~ /mswin|mingw/
@@ -26,4 +14,4 @@ class TestFiddle < Fiddle::TestCase
     end
   end
 
-end
+end if defined?(Fiddle)
