@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 require 'resolv'
 require 'socket'
@@ -53,7 +54,7 @@ class TestResolvDNS < Test::Unit::TestCase
           }
         }
         server_thread = Thread.new {
-          msg, (_, client_port, _, client_address) = timeout(5) {u.recvfrom(4096)}
+          msg, (_, client_port, _, client_address) = Timeout.timeout(5) {u.recvfrom(4096)}
           id, word2, qdcount, ancount, nscount, arcount = msg.unpack("nnnnnn")
           qr =     (word2 & 0x8000) >> 15
           opcode = (word2 & 0x7800) >> 11
@@ -160,7 +161,7 @@ class TestResolvDNS < Test::Unit::TestCase
     # A rase condition here.
     # Another program may use the port.
     # But no way to prevent it.
-    timeout(5) do
+    Timeout.timeout(5) do
       Resolv::DNS.open(:nameserver_port => [[host, port]]) {|dns|
         assert_equal([], dns.getresources("test-no-server.example.org", Resolv::DNS::Resource::IN::A))
       }
@@ -177,4 +178,72 @@ class TestResolvDNS < Test::Unit::TestCase
       end
     end
   end
+
+  def test_resolv_conf_by_command
+    Dir.mktmpdir do |dir|
+      Dir.chdir(dir) do
+        assert_raise(Errno::ENOENT, Errno::EINVAL) do
+          Resolv::DNS::Config.parse_resolv_conf("|echo foo")
+        end
+      end
+    end
+  end
+
+  def test_dots_diffences
+    name1 = Resolv::DNS::Name.create("example.org")
+    name2 = Resolv::DNS::Name.create("ex.ampl.eo.rg")
+    assert_not_equal(name1, name2, "different dots")
+  end
+
+  def test_case_insensitive_name
+    bug10550 = '[ruby-core:66498] [Bug #10550]'
+    lower = Resolv::DNS::Name.create("ruby-lang.org")
+    upper = Resolv::DNS::Name.create("Ruby-Lang.org")
+    assert_equal(lower, upper, bug10550)
+  end
+
+  def test_ipv6_name
+    addr = Resolv::IPv6.new("\0"*16)
+    labels = addr.to_name.to_a
+    expected = (['0'] * 32 + ['ip6', 'arpa']).map {|label| Resolv::DNS::Label::Str.new(label) }
+    assert_equal(expected, labels)
+  end
+
+  def test_ipv6_create
+    ref = '[Bug #11910] [ruby-core:72559]'
+    assert_instance_of Resolv::IPv6, Resolv::IPv6.create('::1')
+    assert_instance_of Resolv::IPv6, Resolv::IPv6.create('::1:127.0.0.1')
+  end
+
+  def test_ipv6_should_be_16
+    ref = '[rubygems:1626]'
+
+    broken_message =
+      "\0\0\0\0\0\0\0\0\0\0\0\1" \
+      "\x03ns2\bdnsimple\x03com\x00" \
+      "\x00\x1C\x00\x01\x00\x02OD" \
+      "\x00\x10$\x00\xCB\x00 I\x00\x01\x00\x00\x00\x00"
+
+    e = assert_raise_with_message(Resolv::DNS::DecodeError, /IPv6 address must be 16 bytes/, ref) do
+      Resolv::DNS::Message.decode broken_message
+    end
+    assert_kind_of(ArgumentError, e.cause)
+  end
+
+  def test_too_big_label_address
+    n = 2000
+    m = Resolv::DNS::Message::MessageEncoder.new {|msg|
+      2.times {
+        n.times {|i| msg.put_labels(["foo#{i}"]) }
+      }
+    }
+    Resolv::DNS::Message::MessageDecoder.new(m.to_s) {|msg|
+      2.times {
+        n.times {|i|
+          assert_equal(["foo#{i}"], msg.get_labels.map {|label| label.to_s })
+        }
+      }
+    }
+    assert_operator(2**14, :<, m.to_s.length)
+  end
 end
