@@ -174,6 +174,7 @@ class TestParse < Test::Unit::TestCase
     end
 
     c = Class.new
+    c.freeze
     assert_nothing_raised(SyntaxError) do
       eval <<-END, nil, __FILE__, __LINE__+1
       if false
@@ -183,7 +184,6 @@ class TestParse < Test::Unit::TestCase
       END
     end
 
-    c = Class.new
     assert_raise(SyntaxError) do
       eval <<-END, nil, __FILE__, __LINE__+1
         $1 &= 1
@@ -484,21 +484,41 @@ class TestParse < Test::Unit::TestCase
   end
 
   def test_string
-    assert_raise(SyntaxError) do
-      eval '"\xg1"'
-    end
+    mesg = 'from the backslash through the invalid char'
 
-    assert_raise(SyntaxError) do
-      eval '"\u{1234"'
-    end
+    e = assert_syntax_error('"\xg1"', /hex escape/)
+    assert_equal('   ^', e.message.lines.last, mesg)
 
-    assert_raise(SyntaxError) do
-      eval '"\M1"'
-    end
+    e = assert_syntax_error('"\u{1234"', 'unterminated Unicode escape')
+    assert_equal('        ^', e.message.lines.last, mesg)
 
-    assert_raise(SyntaxError) do
-      eval '"\C1"'
-    end
+    e = assert_syntax_error('"\u{xxxx}"', 'invalid Unicode escape')
+    assert_equal('    ^', e.message.lines.last, mesg)
+
+    e = assert_syntax_error('"\u{xxxx', 'Unicode escape')
+    assert_pattern_list([
+                          /.*: invalid Unicode escape\n.*\n/,
+                          /    \^/,
+                          /\n/,
+                          /.*: unterminated Unicode escape\n.*\n/,
+                          /    \^/,
+                          /\n/,
+                          /.*: unterminated string.*\n.*\n/,
+                          /        \^/,
+                        ], e.message)
+
+    e = assert_syntax_error('"\M1"', /escape character syntax/)
+    assert_equal(' ^~~', e.message.lines.last, mesg)
+
+    e = assert_syntax_error('"\C1"', /escape character syntax/)
+    assert_equal(' ^~~', e.message.lines.last, mesg)
+
+    src = '"\xD0\u{90'"\n""000000000000000000000000"
+    assert_syntax_error(src, /:#{__LINE__}: unterminated/o)
+
+    assert_syntax_error('"\u{100000000}"', /invalid Unicode escape/)
+    assert_equal("", eval('"\u{}"'))
+    assert_equal("", eval('"\u{ }"'))
 
     assert_equal("\x81", eval('"\C-\M-a"'))
     assert_equal("\177", eval('"\c?"'))
@@ -537,6 +557,10 @@ class TestParse < Test::Unit::TestCase
     assert_nothing_raised(SyntaxError, bug) do
       assert_equal(sym, eval(':"foo\u{0}bar"'))
     end
+    assert_nothing_raised(SyntaxError) do
+      assert_equal(:foobar, eval(':"foo\u{}bar"'))
+      assert_equal(:foobar, eval(':"foo\u{ }bar"'))
+    end
   end
 
   def test_parse_string
@@ -680,15 +704,17 @@ x = __ENCODING__
   def test_invalid_char
     bug10117 = '[ruby-core:64243] [Bug #10117]'
     invalid_char = /Invalid char `\\x01'/
-    x = x = 1
+    x = 1
     assert_in_out_err(%W"-e \x01x", "", [], invalid_char, bug10117)
     assert_syntax_error("\x01x", invalid_char, bug10117)
     assert_equal(nil, eval("\x04x"))
+    assert_equal 1, x
   end
 
   def test_literal_concat
-    x = x = "baz"
+    x = "baz"
     assert_equal("foobarbaz", eval('"foo" "bar#{x}"'))
+    assert_equal("baz", x)
   end
 
   def test_unassignable
@@ -758,9 +784,10 @@ x = __ENCODING__
     $VERBOSE = true
     stderr = $stderr
     $stderr = StringIO.new("")
-    x = x = 1
+    x = 1
     assert_nil eval("x; nil")
     assert_nil eval("1+1; nil")
+    assert_nil eval("1.+(1); nil")
     assert_nil eval("TestParse; nil")
     assert_nil eval("::TestParse; nil")
     assert_nil eval("x..x; nil")
@@ -770,6 +797,7 @@ x = __ENCODING__
     assert_nil eval("true; nil")
     assert_nil eval("false; nil")
     assert_nil eval("defined?(1); nil")
+    assert_equal 1, x
 
     assert_raise(SyntaxError) do
       eval %q(1; next; 2)
@@ -824,10 +852,11 @@ x = __ENCODING__
     end
 
     assert_nothing_raised do
-      x = x = "bar"
+      x = "bar"
       eval <<-END, nil, __FILE__, __LINE__+1
         :"foo#{"x"}baz" ? 1 : 2
       END
+      assert_equal "bar", x
     end
   end
 
@@ -862,8 +891,12 @@ x = __ENCODING__
   def test_unused_variable
     o = Object.new
     assert_warning(/assigned but unused variable/) {o.instance_eval("def foo; a=1; nil; end")}
+    assert_warning(/assigned but unused variable/) {o.instance_eval("def bar; a=1; a(); end")}
     a = "\u{3042}"
     assert_warning(/#{a}/) {o.instance_eval("def foo; #{a}=1; nil; end")}
+    o = Object.new
+    assert_warning(/assigned but unused variable/) {o.instance_eval("def foo; tap {a=1; a()}; end")}
+    assert_warning('') {o.instance_eval("def bar; a=a=1; nil; end")}
   end
 
   def test_named_capture_conflict
@@ -960,6 +993,29 @@ x = __ENCODING__
     assert_syntax_error("    0b\n", /\^/)
   end
 
+  def test_error_def_in_argument
+    assert_separately([], "#{<<-"begin;"}\n#{<<~"end;"}")
+    begin;
+      assert_syntax_error("def f r:def d; def f 0end", /unexpected/)
+    end;
+
+    assert_syntax_error("def\nf(000)end", /^  \^~~/)
+    assert_syntax_error("def\nf(&)end", /^   \^/)
+  end
+
+  def test_method_location_in_rescue
+    bug = '[ruby-core:79388] [Bug #13181]'
+    obj, line = Object.new, __LINE__+1
+    def obj.location
+      #
+      raise
+    rescue
+      caller_locations(1, 1)[0]
+    end
+
+    assert_equal(line, obj.location.lineno, bug)
+  end
+
   def test_negative_line_number
     bug = '[ruby-core:80920] [Bug #13523]'
     obj = Object.new
@@ -980,17 +1036,55 @@ x = __ENCODING__
     end;
   end
 
-  def test_method_location_in_rescue
-    bug = '[ruby-core:79388] [Bug #13181]'
-    obj, line = Object.new, __LINE__+1
-    def obj.location
-      #
-      raise
-    rescue
-      caller_locations(1, 1)[0]
+  def test_unexpected_token_error
+    assert_raise(SyntaxError) do
+      eval('"x"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx')
     end
+  end
 
-    assert_equal(line, obj.location.lineno, bug)
+  def test_unexpected_token_after_numeric
+    assert_raise_with_message(SyntaxError, /^    \^~~\z/) do
+      eval('0000xyz')
+    end
+    assert_raise_with_message(SyntaxError, /^    \^~~\z/) do
+      eval('1.2i1.1')
+    end
+  end
+
+  def test_truncated_source_line
+    e = assert_raise_with_message(SyntaxError, /unexpected tIDENTIFIER/) do
+      eval("'0123456789012345678901234567890123456789' abcdefghijklmnopqrstuvwxyz0123456789 0123456789012345678901234567890123456789")
+    end
+    line = e.message.lines[1]
+    assert_operator(line, :start_with?, "...")
+    assert_operator(line, :end_with?, "...\n")
+  end
+
+  def test_unterminated_regexp_error
+    e = assert_raise(SyntaxError) do
+      eval("/x")
+    end.message
+    assert_match(/unterminated regexp meets end of file/, e)
+    assert_not_match(/unexpected tSTRING_END/, e)
+  end
+
+  def test_lparenarg
+    o = Struct.new(:x).new
+    def o.i(x)
+      self.x = x
+    end
+    o.instance_eval {i (-1.3).abs}
+    assert_equal(1.3, o.x)
+    o.instance_eval {i = 0; i (-1.3).abs}
+    assert_equal(1.3, o.x)
+  end
+
+  def test_serial_comparison
+    assert_warning(/comparison '<' after/) do
+      $VERBOSE = true
+      x = 1
+      eval("if false; 0 < x < 2; end")
+    end
   end
 
   def test_eof_in_def
