@@ -1,4 +1,5 @@
-# $Id: test_scanf.rb 11708 2007-02-12 23:01:19Z shyouhei $
+# frozen_string_literal: false
+# $Id: test_scanf.rb 60261 2017-10-21 12:34:13Z nobu $
 #
 # scanf for Ruby
 #
@@ -7,6 +8,7 @@
 
 require 'scanf.rb'
 require 'test/unit'
+require 'tempfile'
 
 # Comment out either of these lines to skip those tests.
 
@@ -15,7 +17,7 @@ class TestIOScanf < Test::Unit::TestCase;end
 
 module ScanfTests
 
-def tests
+  def tests
     [
 
 # Scratchpad
@@ -40,12 +42,9 @@ def tests
       [ "%d\n%[abc]", "\neabc", [] ],
       [ "%d%[^abc]", "ghiabc", [ ] ],
       [ "%d%[abc]", "abc", [] ],
-      [ "%*d %*3d %*s", "123 +456 abc", [] ],
       [ "%d%s", "", [] ],
       [ "%d%s", "blah 123 string", [] ],
       [ "%[\n]", "abc\n", [] ],
-      [ "%[\n]", "abc\n", [] ],
-      [ "%[\n]", "abc\n", [] ],
       [ "%f", "x", [] ],
       [ "%f", "z", [] ],
       [ "%f", "z3.2534", [] ],
@@ -68,7 +67,6 @@ def tests
       [ "%3c", "a\nb", [ "a\nb" ] ],
       [ "%3c", "a\nbcd", [ "a\nb" ] ],
       [ "%c\n\n", "x\n\n", [ "x" ] ],
-      [ "%c\n\n", "x\n\n", [ "x" ] ],
       [ "%c", "\n", [ "\n" ] ],
       [ "%c", "x\n", [ "x" ] ],
       [ "%2c", " 123", [" 1"] ],
@@ -79,7 +77,6 @@ def tests
       [ "%5c", "a\nb\n\n", [ "a\nb\n\n" ] ],
       [ "%6c", "a\nb\n\nx", [ "a\nb\n\nx" ] ],
       [ "%5c", "ab\ncd", [ "ab\ncd" ] ],
-      [ "%5c", "a\nb\n\n", [ "a\nb\n\n" ] ],
 
 # Testing 'o'
       [ "%3o", "0xz", [0] ],
@@ -87,7 +84,6 @@ def tests
 # Testing 'd'
       [ "%d", "\n123", [ 123 ] ],
       [ "%d", "\n\n123", [ 123 ] ],
-      [ "%d", "\n123", [ 123 ] ],
       [ "%1d", "2", [2] ],
 
 # Mixed tests
@@ -97,7 +93,6 @@ def tests
 #   various mixed specifiers
 
       [ "%[^\\w]%c", "...1", [ "...", "1"] ],
-      [ '%[^\\w]%c', "...1", [ "...", "1"] ],
       [ "%[abc\n]%d", "a\n\nb\n\nc   123", [ "a\n\nb\n\nc", 123 ] ],
       [ "%[abc\n]%d", "a\n\nb\n\nc \t  123", [ "a\n\nb\n\nc", 123 ] ],
       [ "%[abc\t]%d", "a\t\tb\t\tc   123", [ "a\t\tb\t\tc", 123 ] ],
@@ -114,7 +109,6 @@ def tests
       [ "%d%3[^abc]", "123defdef", [ 123, "def" ] ],
       [ "%d%3[^abc] ", "123defdef   ", [ 123, "def" ] ],
       [ "%d%3[^abc]ghi", "123defghi", [ 123, "def" ] ],
-      [ "%d%3[^abc]", "123defdef", [ 123, "def" ] ],
       [ "%d%3[^abc]", "123adefdef", [ 123 ] ],
       [ "%d%3[^abc]", "123deafdef", [ 123, "de" ] ],
       [ "%d%3[^abc\n]", "123de\nf", [ 123, "de" ] ],
@@ -141,9 +135,7 @@ def tests
       [ "%c\n\n%c", "x\n\ny", [ "x", "y" ] ],
       [ "%s%d%d", "abc\n123\n456", [ "abc", 123, 456 ] ],
       [ "%3s%c%3c%d", "1.2x\n\ny123", [ "1.2", "x", "\n\ny", 123 ] ],
-      [ "%f%3c", "1.2x\ny", [ 1.2, "x\ny"] ],
       [ "%c\n%c", "x\n\ny", [ "x", "y" ] ],
-      [ "%c\n\n%c", "x\n\ny", [ "x", "y" ] ],
       [ "%c  %c", "x\n\ny", [ "x", "y" ] ],
       [ "%s\n\n%c", "x\n\ny", [ "x", "y" ] ],
       [ "%s\n\n%s", "x\n\ny", [ "x", "y" ] ],
@@ -151,10 +143,6 @@ def tests
       [ "%d\n%d", "23\n\n45", [ 23, 45 ] ],
       [ "%c\n\n%c", "x y", [ "x", "y" ] ],
       [ "%c%c", "x\n\ny", [ "x", "\n" ] ],
-      [ "%c\n%c", "x y", [ "x", "y" ] ],
-      [ "%c\t%c", "x y", [ "x", "y" ] ],
-      [ "%s\n%s", "x y", [ "x", "y" ] ],
-      [ "%s%s\n", "x y", [ "x", "y" ] ],
       [ "%c%c", "x\n", [ "x", "\n" ] ],
       [ "%d%c%c%d", "345   678", [ 345, " ", " ", 678] ],
       [ "%d   %c%s", "123   x  hello", [123, "x", "hello"] ],
@@ -178,15 +166,14 @@ def tests
       [ "%3d%3x", "12abc", [12, "abc".hex] ],
       [ "%s%i%d", "hello +0xdef 123", [ "hello", "def".hex, 123] ],
       [ "%s%i%d", "hello -0xdef 123", [ "hello", -"def".hex, 123] ],
-      [ "%s%i%i%i", "hello 012 -012 100", [ "hello", 10, -10, 100 ] ],
-      [ "%s%i%i%i", "hello 012 0x12 100", [ "hello", 10, 18, 100 ] ],
+      [ "%s%i%i%i%i", "hello 012 -012 100 1", [ "hello", 10, -10, 100, 1 ] ],
+      [ "%s%i%i%i%i", "hello 012 0x12 100 1", [ "hello", 10, 18, 100, 1 ] ],
       [ "%s%5i%3i%4i", "hello 0x123 123 0123", [ "hello", "0x123".hex, 123,"0123".oct] ],
       [ "%s%3i%4i", "hello 1230123", [ "hello", 123,"0123".oct] ],
       [ "%s%3i", "hello 1230", [ "hello", 123] ],
       [ "%s%5x%d", "hello 0xdef 123", [ "hello", "def".hex, 123] ],
       [ "%s%6x%d", "hello +0xdef 123", [ "hello", "def".hex, 123] ],
       [ "%s%6x%d", "hello -0xdef 123", [ "hello", -"def".hex, 123] ],
-      [ "%s%6x%d", "hello -0xdef 123", [ "hello", -"def".hex, 123] ],
       [ "%s%4x%d", "hello -def 123", [ "hello", -"def".hex, 123] ],
       [ "%s%3x%d", "hello def 123", [ "hello", "def".hex, 123] ],
       [ "%s%x%d", "hello -def 123", [ "hello", -"def".hex, 123] ],
@@ -200,7 +187,6 @@ def tests
       [ "%f %d %s", "12.3e23 45 string", ["12.3e23".to_f, 45, "string"] ],
       [ "%f %d %s", "12.3e+23 45 string", ["12.3e23".to_f, 45, "string"] ],
       [ "%f %d %s", "12.3e-23 45 string", ["12.3e-23".to_f, 45, "string"] ],
-      [ "%f %d %s", "12.3e23 45 string", ["12.3e23".to_f, 45, "string"] ],
       [ "%f %d %s", "-12.3e-23 45 string", ["-12.3e-23".to_f, 45, "string"] ],
       [ "%f %d %s", "12.e23 45 string", ["12.e23".to_f, 45, "string"] ],
       [ "%5f %d %s", "1.2e23 string", ["1.2e2".to_f, 3, "string"] ],
@@ -233,14 +219,12 @@ def tests
       [ "%d%s", "123", [123] ],
       [ "%s%d", "abc", ["abc"] ],
       [ "%f%x", "3.2e45x", ["3.2e45x".to_f] ],
-      [ "%s%d", "abc", ["abc"] ],
       [ "%*5f%d %d %s", "1.2e23 45 string", [3, 45, "string"] ],
       [ "%5f%*d %d %s", "1.2e23 45 string", ["1.2e2".to_f, 45, "string"] ],
       [ "%*5f%*d %*d %s", "1.2e23 45 string", ["string"] ],
       [ "%f %*d %s", "12.e23 45 string", ["12.e23".to_f, "string"] ],
-      [ "%5f %d %s", "1.2e23 string", ["1.2e2".to_f, 3, "string"] ],
-      [ "%s %f %s %d %x%c%c%c%c", 
-        "float: 1.2e23 dec/hex: 135a23 abc", 
+      [ "%s %f %s %d %x%c%c%c%c",
+        "float: 1.2e23 dec/hex: 135a23 abc",
         ["float:", "1.2e23".to_f, "dec/hex:", 135, "a23".hex, " ", "a", "b", "c" ] ],
 
 # Testing 's'
@@ -256,8 +240,6 @@ def tests
       [ "%[\n]", "\nabc", [ "\n" ] ],
       [ "%[\n\t]", "\t\n", [ "\t\n" ] ],
       [ "%[a-f]", "abczef", [ "abc" ] ],
-      [ "%d%3[abc]", "123 aaab", [ 123 ] ],
-      [ "%d%3[^abc]", "123adefdef", [ 123 ] ],
       [ "%d%3[[:lower:]] %f", "123ade1.2", [ 123,"ade",1.2 ] ],
       [ "%d%3[[:lower:]] %f", "123ad1.2", [ 123,"ad",1.2 ] ],
       [ "%d%3[[:lower:]] %f", "123 ad1.2", [ 123 ] ],
@@ -268,43 +250,56 @@ def tests
       [ "%[[:upper:]]", "ABCdefGHI", [ "ABC" ] ],
 
 # Testing 'f'
-      [ "%2f", "x", [0.0] ],  # width-floats match anything (by design)
-      [ "%f", "1.23e45", [1.23e+45] ],
-      [ "%f", "3.25ee", [3.25] ],
-      [ "%f", "3..25", [3.0] ],
-      [ "%f", "+3.25", [3.25] ],
-      [ "%f", "+3.25e2", [325.0] ],
+      [ "%2f", "x", [] ],
+      [ "%F", "1.23e45", [1.23e+45] ],
+      [ "%e", "3.25ee", [3.25] ],
+      [ "%E", "3..25", [3.0] ],
+      [ "%g", "+3.25", [3.25] ],
+      [ "%G", "+3.25e2", [325.0] ],
       [ "%f", "3.z", [3.0] ],
-     ]
+      [ "%a", "0X1P+10", [1024.0] ],
+      [ "%a", "0X1P10", [1024.0] ],
+      [ "%A", "0x1.deadbeefp+99", [1.1851510441583988e+30] ],
+
+# Testing embedded matches including literal '[' behavior
+      [",%d,%f", ",10,1.1", [10,1.1] ],
+      [" ,%d,%f", " ,10,1.1", [10,1.1] ],
+      ["[%d,%f", "[10,1.1", [10,1.1] ],
+      [" [%d,%f", " [10,1.1", [10,1.1] ],
+
+    ]
+  end
+
+  def each_test
+    self.tests.each do |test|
+      format, string, = test
+      yield test, "#{string.dump}(#{format.dump})"
     end
   end
+end
 
 class TestStringScanf
   include Scanf
   extend ScanfTests
 
-  i = 1
-  self.tests.each do |test|
+  self.each_test do |test, i|
     define_method("test_#{i}") do ||
       assert_equal(test[2], test[1].scanf(test[0]))
-      end
-    i += 1
+    end
   end
 end
 
 class TestIOScanf
   include Scanf
   extend ScanfTests
-  
-  i = 1
-  self.tests.each do |test|
+
+  self.each_test do |test, i|
     define_method("test_#{i}") do ||
-      File.open("iotest.dat", "w") {|fh| fh.print test[1]}
-      File.open("iotest.dat", "r") { |fh|
+      Tempfile.create("iotest.dat") do |fh|
+        fh.print test[1]
+        fh.rewind
         assert_equal(test[2], fh.scanf(test[0]))
-      }
-      File.delete("iotest.dat")
+      end
     end
-    i += 1
   end
 end
