@@ -1,3 +1,6 @@
+# Used as part of the "make parse" Makefile target.
+# See common.mk for details.
+
 $file = ARGV[0]
 $str  = ARGF.read.sub(/^__END__.*\z/m, '')
 puts '# ' + '-' * 70
@@ -6,7 +9,7 @@ puts '# ' + '-' * 70
 puts $str
 puts '# ' + '-' * 70
 
-$parsed = VM::InstructionSequence.compile_file($file)
+$parsed = RubyVM::InstructionSequence.compile_file($file)
 puts "# disasm result: "
 puts '# ' + '-' * 70
 puts $parsed.disasm
